Type: Package
Package: cleanr
Title: Helps You to Code Cleaner
Version: 1.4.0
Authors@R: 
    person(given = "Andreas Dominik",
           family = "Cullmann",
           role = c("aut", "cre"),
           email = "fvafrcu@mailbox.org")
Description: Check your R code for some of the most common
    layout flaws.  Many tried to teach us how to write code less dreadful,
    be it implicitly as B. W. Kernighan and D. M. Ritchie (1988)
    <ISBN:0-13-110362-8> in 'The C Programming Language' did, be it
    explicitly as R.C. Martin (2008) <ISBN:0-13-235088-2> in 'Clean Code:
    A Handbook of Agile Software Craftsmanship' did.  So we should check
    our code for files too long or wide, functions with too many lines,
    too wide lines, too many arguments or too many levels of nesting.
    Note: This is not a static code analyzer like pylint or the like.
    Checkout <https://cran.r-project.org/package=lintr> instead.
License: BSD_2_clause + file LICENSE
URL: https://gitlab.com/fvafrcu/cleanr
Depends: R (>= 3.3.0)
Imports: checkmate, fritools, pkgload, rprojroot
Suggests: devtools, rasciidoc, rmarkdown, RUnit, testthat, usethis
VignetteBuilder: rasciidoc
Encoding: UTF-8
RoxygenNote: 7.2.3
Collate: 'utils.R' 'internals.R' 'checks.R' 'cleanr-package.R'
        'options.R' 'throw.R' 'wrappers.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2023-06-15 21:28:59 UTC; qwer
Author: Andreas Dominik Cullmann [aut, cre]
Maintainer: Andreas Dominik Cullmann <fvafrcu@mailbox.org>
Repository: CRAN
Date/Publication: 2023-06-16 08:30:24 UTC
Built: R 4.6.0; ; 2026-04-26 12:10:38 UTC; unix

# Generated by roxygen2: do not edit by hand

export(check_directory)
export(check_file)
export(check_file_layout)
export(check_file_length)
export(check_file_width)
export(check_function_layout)
export(check_functions_in_file)
export(check_line_width)
export(check_nesting_depth)
export(check_num_arguments)
export(check_num_lines)
export(check_num_lines_of_code)
export(check_package)
export(check_return)
export(gco)
export(get_cleanr_options)
export(is_not_false)
export(load_internal_functions)
export(set_cleanr_options)

[![pipeline status](https://gitlab.com/fvafrcu/cleanr/badges/master/pipeline.svg)](https://gitlab.com/fvafrcu/cleanr/-/commits/master)    
[![coverage report](https://gitlab.com/fvafrcu/cleanr/badges/master/coverage.svg)](https://gitlab.com/fvafrcu/cleanr/-/commits/master)
<!-- 
    [![Build Status](https://travis-ci.org/fvafrcu/cleanr.svg?branch=master)](https://travis-ci.org/fvafrcu/cleanr)
    [![Coverage Status](https://codecov.io/github/fvafrcu/cleanr/coverage.svg?branch=master)](https://codecov.io/github/fvafrcu/cleanr?branch=master)
-->
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/cleanr)](https://cran.r-project.org/package=cleanr)
[![RStudio_downloads_monthly](https://cranlogs.r-pkg.org/badges/cleanr)](https://cran.r-project.org/package=cleanr)
[![RStudio_downloads_total](https://cranlogs.r-pkg.org/badges/grand-total/cleanr)](https://cran.r-project.org/package=cleanr)

# cleanr
Check your R code for some of the most common layout flaws.
<!-- README.md is generated from README.Rmd. Please edit that file -->



## Introduction
After installation, please read the help page:

```r
help("cleanr-package", package = "cleanr")
```

```
#> Helps You to Code Cleaner
#> 
#> Description:
#> 
#>      Check your R code for some of the most common layout flaws.
#> 
#> Details:
#> 
#>      Many tried to teach us how to write code less dreadful, be it
#>      implicitly as B. W. Kernighan and D. M. Ritchie in The C
#>      Programming Language did, be it explicitly as R.C. Martin in Clean
#>      Code: A Handbook of Agile Software Craftsmanship did.
#> 
#>      So we should check our code for files too long or wide, functions
#>      with too many lines, too wide lines, too many arguments or too
#>      many levels of nesting.
#> 
#> Note:
#> 
#>      This is not a static code analyzer like pylint or the like. If
#>      you're looking for a static code analyzer, check out lintr
#>      (<https://cran.r-project.org/package=lintr>).
#> 
#> See Also:
#> 
#>      Packages 'codetools'
#>      (<https://cran.r-project.org/package=codetools>), 'formatR'
#>      (<https://cran.r-project.org/package=formatR>) and 'lintr'
#>      (<https://cran.r-project.org/package=lintr>).
```

## Installation

You can install cleanr from gitlab via:


```r
if (! require("remotes")) install.packages("remotes")
remotes::install_gitlab("fvafrcu/cleanr")
```

#' Test if an Object is not False
#'
#' Sometimes you need to know whether or not an object exists and is not set to
#' FALSE (and possibly not NULL).
#'
#' Pass an environment if you call the function elsewhere than from
#' \code{\link{.GlobalEnv}}.
#'
#' @param object The object to be tested.
#' @param null_is_false [boolean(1)]\cr Should NULL be treated as FALSE?
#' @param ... Parameters passed to \code{\link{exists}}. See Details and
#' Examples.
#' @return TRUE if the object is set to something different than FALSE, FALSE
#' otherwise.
#' @keywords internal
#' @export
#' @examples
#' a  <- 1
#' is_not_false(a)
#' f <- function() {
#'     a <- NULL
#'     should_be_true <- ! is_not_false(a, null_is_false = TRUE,
#'                                       where = environment())
#'     return(should_be_true)
#' }
#' print(f())
is_not_false <- function(object, null_is_false = TRUE, ...) {
    checkmate::qassert(null_is_false, "B1")
    condition <- exists(deparse(substitute(object)), ...)
    if (isTRUE(null_is_false)) {
        condition <- condition && ! is.null(object) && object != FALSE
    } else {
        condition <- condition && (is.null(object) || object != FALSE)
    }
    if (condition)
        result <- TRUE
    else
        result <- FALSE
    return(result)
}

#' Load a Package's Internals
#'
#' Load objects not exported from a package's namespace.
#'
#' The files to be checked get sourced, which means they have to contain R code
#' producing no errors. If we want to check the source code of a package, we
#' need to load the package \emph{and} be able to run all its internals in our
#' environment.
#'
#' @param package [character(1)]\cr The name of the package as a string.
#' @param ... Arguments passed to \code{\link{ls}}, all.names = TRUE could be a
#' good idea.
#' @seealso \code{\link[codetools:checkUsageEnv]{checkUsageEnv in codetools}}.
#' @return invisible(TRUE)
#' @export
#' @keywords internal
#' @examples
#' load_internal_functions("cleanr")
load_internal_functions <- function(package, ...) {
    checkmate::qassert(package, "S1")
    library(package, character.only = TRUE)
    exported_names <- ls(paste("package", package, sep = ":"), ...)
    is_exported_name_function <- vapply(exported_names,
                                        function(x) is.function(get(x)), TRUE)
    exported_functions <- exported_names[is_exported_name_function]
    package_namespace <- asNamespace(package)
    package_names <- ls(envir = package_namespace)
    is_package_name_function <-
        vapply(package_names,
               function(x) is.function(get(x, envir = package_namespace)),
               TRUE)
    package_functions <- package_names[is_package_name_function]
    internal_functions <- setdiff(package_functions, exported_functions)
    for (name in internal_functions) {
        assign(name, get(name, envir = package_namespace, inherits = FALSE),
               envir = parent.frame())
    }
    return(invisible(TRUE))
}

cleanr::set_cleanr_options(reset = TRUE)

path <- system.file("runit_tests", "runit_wrappers.R", package = "cleanr")
print(cleanr::check_file_layout(path))

cleanr::get_cleanr_options(flatten_list = FALSE)

## cleanr::check_file_layout(path, max_file_width = 79)

print(tools::assertCondition(cleanr::check_file_layout(path, 
                                                     max_file_width = 79), 
                             c("cleanr", "error",  "condition"))
)

cleanr::set_cleanr_options(max_file_width = 75, 
                           max_file_length = 20)
print(tools::assertCondition(cleanr::check_file_layout(path), 
                             c("cleanr", "error",  "condition"))
)

cleanr::set_cleanr_options(reset = TRUE)
path <- system.file("source", "R", "checks.R", package = "cleanr")
print(cleanr::check_file_layout(path)) 

print(tools::assertCondition(cleanr::check_file_layout(path, 
                                                       max_file_length = 100), 
                             c("cleanr", "error",  "condition"))
)

print(suppressWarnings(cleanr::check_functions_in_file(path)))

print(tools::assertCondition(suppressWarnings(cleanr::check_functions_in_file(path,
                                                                        max_num_arguments = 1)), 
                       c("cleanr", "error",  "condition"))
)

print(suppressWarnings(cleanr::check_file(path)))

path <- system.file("runit_tests", package = "cleanr")
print(suppressWarnings(cleanr::check_directory(path,
                                               check_return = FALSE)))

path <- system.file("source", "R", package = "cleanr")
cleanr::load_internal_functions("cleanr")
print(tools::assertCondition(suppressWarnings(cleanr::check_directory(path)), 
                       c("cleanr", "error",  "condition"))
)

cleanr::set_cleanr_options(reset = TRUE)
cleanr::load_internal_functions("cleanr")
path <- system.file("source", "R", "wrappers.R", package = "cleanr")

# will produce a condition:
r <- tools::assertCondition(cleanr::check_functions_in_file(path, 
                                                            check_return = FALSE),
                            c("cleanr", "error",  "condition"))
print(r)
# set argument to disable the check causing the condition:
r <- cleanr::check_functions_in_file(path, check_return = FALSE, 
                                     max_num_arguments = FALSE)
print(r)

co <- get_cleanr_options(flatten_list = FALSE)
co <- lapply(co, function(x) x == FALSE)
options("cleanr" = list(cleanr = co))
get_cleanr_options() 

path <- system.file("source", "R", package = "cleanr")
cleanr::load_internal_functions("cleanr")
print(cleanr::check_directory(path)) 

#' Throw a Condition
#'
#' Throws a condition of class c("error", "cleanr", "condition").
#'
#' We use this condition as an error dedicated to \pkg{ cleanr.}
#'
#' @param message_string The message to be thrown.
#' @param system_call The call to be thrown.
#' @param ... Arguments to be passed to
#' \code{\link[base:structure]{base::structure}}.
#' @return The function does never return anything, it stops with a
#' condition of class c("error", "cleanr", "condition").
#' @keywords internal
#' @examples
#' tryCatch(cleanr:::throw("Hello error!"), cleanr = function(e) return(e))
throw <- function(message_string, system_call = sys.call(-1), ...) {
    condition <- structure(class = c("error", "cleanr", "condition"),
                           list(message = message_string, call = system_call),
                           ...)
    stop(condition)
}

#' Set Options for \pkg{cleanr}
#'
#' A convenience function for \code{\link{options}}.
#'
#' \pkg{cleanr} loads a couple of options as defaults for its
#' functions.
#' The defaults are stored in a list element of \code{\link{options}}.
#' All checks (see \code{\link{function_checks}} and \code{\link{file_checks}})
#' can be disabled by setting the corresponding option list item to \code{NULL}
#' or \code{FALSE}.
#' @param overwrite [boolean(1)]\cr Overwrite options already set? Is set to
#' \code{FALSE} on package loading to ensure your previously set \pkg{cleanr}
#' options won't get overridden. Just ignore that argument.
#' @param reset [boolean(1)]\cr Reset all \pkg{cleanr} options to the package's
#' defaults?
#' @param ... See \code{\link{options}}. See \code{\link{function_checks}} and
#' \code{\link{file_checks}} for options to be set with \pkg{cleanr}.
#' Use \code{\link{get_cleanr_options}} to get the current values.
#' \pkg{cleanr}'s standard defaults are:
#' \Sexpr[results=verbatim]{cat(paste(names(cleanr::gco()),
#'                                    cleanr::gco(flatten_list = FALSE),
#'                                    sep = "=", collapse = "\n"), "\n")}
#' @return \code{\link[base:invisible]{Invisibly}} \code{\link{TRUE}}.
#' @family option wrappers
#' @export
#' @examples
#' # R.C. Martin's Clean Code recommends monadic argument lists.
#' cleanr::set_cleanr_options(max_num_arguments = 1)
#' # R.C. Martin's Clean Code recommends functions less than 20 lines long.
#' cleanr::set_cleanr_options(max_lines = 30, max_lines_of_code = 20)
#' # same as above:
#' cleanr::set_cleanr_options(list(max_lines = 30, max_lines_of_code = 20))
#' cleanr::get_cleanr_options(flatten_list = TRUE)
#' # we delete all options and set some anew
#' options("cleanr" = NULL)
#' options("cleanr" = list(max_lines = 30, max_lines_of_code = 20))
#' # fill the missing options with the package's defaults:
#' cleanr::set_cleanr_options(overwrite = FALSE)
#' cleanr::get_cleanr_options(flatten_list = TRUE)
#' # reset to the package's defaults:
#' cleanr::set_cleanr_options(reset = TRUE)
#' cleanr::get_cleanr_options(flatten_list = TRUE)
set_cleanr_options <- function(..., reset = FALSE, overwrite = TRUE) {
    checkmate::qassert(reset, "B1")
    checkmate::qassert(overwrite, "B1")
    defaults <- list(max_file_width = 80, max_file_length = 300,
                     max_lines = 65, max_lines_of_code = 50,
                     max_num_arguments = 5, max_nesting_depth = 3,
                     max_line_width = 80, check_return = TRUE)
    option_list <- list(...)
    if (is.null(getOption("cleanr")) || reset)
        options("cleanr" = defaults)
    else {
        set_options <- getOption("cleanr")
        if (overwrite) {
            options("cleanr" = utils::modifyList(set_options, option_list))
        } else {
            if (length(option_list) == 0)
                option_list <- defaults
            is_option_unset <- !(names(option_list) %in% names(set_options))
            if (any(is_option_unset))
                options("cleanr" = append(set_options,
                                              option_list[is_option_unset]))
        }
    }
    max_lines <- get_cleanr_options("max_lines", flatten_list = TRUE,
                                    remove_names = TRUE)
    max_lines_of_code <- get_cleanr_options("max_lines_of_code",
                                           flatten_list = TRUE,
                                           remove_names = TRUE)
    if (max_lines < max_lines_of_code) {
        set_cleanr_options("max_lines" = max_lines_of_code)
        warning(paste("maximum number of lines was less than maximum number of",
                      "lines of code, resetting the former to the latter."))
    }
    return(invisible(TRUE))
}

#' Get options for \pkg{cleanr}
#'
#' A convenience function for \code{\link{getOption}}.
#'
#' @param ... See \code{\link{getOption}}.
#' @param remove_names [boolean(1)]\cr Remove the names?
#' @param flatten_list [boolean(1)]\cr Return a vector?
#' @return a (possibly named) list or a vector.
#' @export
#' @family option wrappers
#' @examples
#' cleanr::get_cleanr_options("max_lines")
#' cleanr::get_cleanr_options("max_lines", remove_names = TRUE)
#' cleanr::get_cleanr_options("max_lines", flatten_list = TRUE)
#' cleanr::get_cleanr_options("max_lines", flatten_list = TRUE,
#'                            remove_names = TRUE)
#' cleanr::get_cleanr_options(flatten_list = TRUE, remove_names = TRUE)
#' cleanr::get_cleanr_options(c("max_lines", "max_lines_of_code"))
get_cleanr_options <- function(..., remove_names = FALSE, flatten_list = TRUE) {
    checkmate::qassert(remove_names, "B1")
    checkmate::qassert(flatten_list, "B1")
    if (missing(...)) {
        option_list <- getOption("cleanr")
    } else {
        option_names <- as.vector(...)
        options_set <- getOption("cleanr")
        option_list  <- options_set[names(options_set) %in% option_names]
    }
    if (flatten_list) option_list <-  unlist(option_list)
    if (remove_names) names(option_list)  <- NULL
    return(option_list)
}

#' @rdname get_cleanr_options
#' @export
#' @family option wrappers
#' @note \code{gco} is just an alias for \code{get_cleanr_options}.
gco <- get_cleanr_options

methods::setMethod(f = "auc", methods::signature(object = "bincap"),
          function(object) {
              n0 <- sum(object@response == object@true)
              n1 <- sum(object@response != object@true)
              s0 <- sum(rank(object@predicted, ties.method = "average",
                             na.last = TRUE) * (object@response == object@true))
              result <- (s0 - n0 * (n0 + 1) / 2) / (n0 * n1)
              return(result)
          }
          )

if (interactive()) pkgload::load_all()
function_file <- system.file("runit_tests", "source", "R", "functions.R",
                             package = "cleanr")
source(function_file)

test_check_function_layout <- function() {
    RUnit::checkTrue(check_function_layout(passing_function))
    RUnit::checkException(check_function_layout(failing_function,
                                                max_line_width = 10))
}

test_check_file_layout <- function() {
    RUnit::checkTrue(check_file_layout(function_file, max_file_width = 100))
    RUnit::checkException(check_file_layout(function_file, max_file_width = 10))
}

test_check_functions_in_file <- function() {
    RUnit::checkTrue(check_functions_in_file(function_file,
                                             max_nesting_depth = 4,
                                             max_line_width = 100,
                                             check_return = FALSE))
    RUnit::checkException(check_functions_in_file(function_file))
}

test_check_file <- function() {
    RUnit::checkTrue(check_file(function_file,
                                             max_nesting_depth = 4,
                                             max_line_width = 100,
                                             max_file_width = 100,
                                             check_return = FALSE))
    RUnit::checkException(check_file(function_file))
    RUnit::checkException(check_file(function_file, silly_arguemnt = "FIFI"))
}

test_check_directory <- function() {
    RUnit::checkTrue(check_directory(system.file("runit_tests", "source", "R",
                                                 package = "cleanr"),
                                     max_nesting_depth = 4,
                                     max_line_width = 100,
                                     max_file_width = 100,
                                     check_return = FALSE))
    RUnit::checkException(check_directory(system.file("runit_tests", "source",
                                                      "R", package = "cleanr")))
}

methods::setClass(Class = "cap",
         representation = methods::representation(response = "factor",
                                                  "VIRTUAL"),
         prototype = methods::prototype(response = factor()),
         validity = function(object) {
         }
         )

methods::setClass(Class = "bincap",
         contains = "cap",
         representation = methods::representation(predicted = "numeric",
                                         true = "character"),
         prototype = methods::prototype(predicted = numeric(),
                                        true = character()),
         validity = function(object) {
             if (length(object@response) != length(object@predicted))
              return("response and predicted must have the same number of obs.")
             if (any(object@predicted < 0, na.rm = TRUE) ||
                 any(object@predicted > 1, na.rm = TRUE))
              return("probabilities should be in [0,1].")
             if (length(object@true) > 1)
              return("give a single character for the 'true'/'presence' class.")
             if (length(levels(object@response)) > 2)
              return("response has to be a two class factor.")
         }
         )

test_options <- function() {
    set_cleanr_options(max_file_width = 100)
    RUnit::checkIdentical(get_cleanr_options("max_file_width"),
                          c("max_file_width" = 100))
    RUnit::checkIdentical(get_cleanr_options("max_file_width",
                                             flatten_list = FALSE),
                          list("max_file_width" = 100))
    RUnit::checkIdentical(get_cleanr_options("max_file_width",
                                             remove_names = TRUE), c(100))
    RUnit::checkIdentical(get_cleanr_options("max_file_width",
                                             remove_names = TRUE,
                                             flatten_list = FALSE), list(100))
    options("cleanr" = list(max_lines = 30, max_lines_of_code = 20))
    set_cleanr_options()
    RUnit::checkIdentical(get_cleanr_options(),
                          c(max_lines = 30, max_lines_of_code = 20))
    set_cleanr_options(max_lines = 100, max_line_width = 20, overwrite = FALSE)
    RUnit::checkIdentical(get_cleanr_options(),
                          c(max_lines = 30, max_lines_of_code = 20,
                            max_line_width = 20))
    set_cleanr_options(max_lines = 40, max_line_width = 80, overwrite = TRUE)
    RUnit::checkIdentical(get_cleanr_options(),
                          c(max_lines = 40, max_lines_of_code = 20,
                            max_line_width = 80))
    options("cleanr" = list(max_lines = 30, max_lines_of_code = 20))
    set_cleanr_options(max_lines_of_code = 40, overwrite = TRUE)
    RUnit::checkIdentical(get_cleanr_options(),
                          c(max_lines = 40, max_lines_of_code = 40))
    set_cleanr_options(reset = TRUE)
}

test_get_function_body <- function() {
    assign("f", function() print(3))
    result <- cleanr:::get_function_body(f)
    expectation <- "print(3)"
    print(result)
    RUnit::checkIdentical(result, expectation)
}

#!/usr/bin/Rscript --vanilla
library("cleanr")
file_name <- system.file("source", "R", "utils.R", package = "cleanr")

message("If a file passes all checks, cleanr returns invisibly TRUE")
print(suppressWarnings(check_file(file_name)))

message("Show current options:")
get_cleanr_options(flatten_list = TRUE)

message("Change an option to procude conditions:")
set_cleanr_options(max_file_width = get_cleanr_options("max_file_width") - 1)

message("Catch cleanr conditions:")
print(tryCatch(suppressWarnings(check_file(file_name)),
               cleanr = function(e) return(e)))

test_is_not_false <- function() {
    # Begin Exclude Linting
    assign("a", FALSE, envir = .GlobalEnv)
    RUnit::checkTrue(! is_not_false(a, envir = .GlobalEnv))
    RUnit::checkTrue(! is_not_false(a, null_is_false = FALSE,
                                    envir = .GlobalEnv))

    assign("a", TRUE, envir = .GlobalEnv)
    RUnit::checkTrue(is_not_false(a, envir = .GlobalEnv))
    RUnit::checkTrue(is_not_false(a, null_is_false = FALSE, envir = .GlobalEnv))

    assign("a", 3, envir = .GlobalEnv)
    RUnit::checkTrue(is_not_false(a, envir = .GlobalEnv))
    RUnit::checkTrue(is_not_false(a, null_is_false = FALSE, envir = .GlobalEnv))

    assign("a", NULL, envir = .GlobalEnv)
    RUnit::checkTrue(! is_not_false(a, envir = .GlobalEnv))
    RUnit::checkTrue(is_not_false(a, null_is_false = FALSE, envir = .GlobalEnv))
    rm("a", envir = .GlobalEnv)

    RUnit::checkTrue(! is_not_false(a, envir = .GlobalEnv))
    RUnit::checkTrue(! is_not_false(a, null_is_false = FALSE,
                                    envir = .GlobalEnv))
    # End Exclude Linting
}

#' Helps You to Code Cleaner
#'
#' Check your R code for some of the most common layout flaws.
#'
#' Many tried to teach us how to write code less dreadful, be it implicitly as
#' B. W. Kernighan and D. M. Ritchie in \cite{The C Programming Language} did,
#' be it explicitly as
#' R.C. Martin in \cite{Clean Code: A Handbook of Agile Software Craftsmanship}
#' did.
#'
#' So we should check our code for files too long or wide, functions with too
#' many lines, too wide lines, too many arguments or too many levels of nesting.
#' @note This is not a static code analyzer like pylint or the like. If you're
#' looking for a static code analyzer, check out lintr
#' (\url{https://cran.r-project.org/package=lintr} or
#' \url{https://github.com/jimhester/lintr}).
#' @seealso Packages
#' \pkg{codetools}
#' (\url{https://cran.r-project.org/package=codetools}),
#' \pkg{formatR}
#' (\url{https://cran.r-project.org/package=formatR})
#' and
#' \pkg{lintr}
#' (\url{https://cran.r-project.org/package=lintr}).
#'
#' @name cleanr-package
#' @aliases cleanr-package
#' @docType package
#' @keywords package
NULL

if (interactive()) pkgload::load_all()

test_s4_no_generic <- function() {
    file <- system.file("runit_tests", "source", "R_s4", "auc-methods.R",
                        package = "cleanr")
    # FIXME: what the hell is going on?!
    # This _should_ throw an exception, but does so only interactively?
    # But coverage is there because of test_s4_dircetory in batch!
    if (interactive()) {
        RUnit::checkException(cleanr::check_file(file))
    } else {
        RUnit::checkTrue(cleanr::check_file(file))
    }
}

test_s4_known_generic <- function() {
    file <- system.file("runit_tests", "source", "R_s4", "AllGenerics.R",
                        package = "cleanr")
    source(file)
    file <- system.file("runit_tests", "source", "R_s4", "auc-methods.R",
                        package = "cleanr")
    RUnit::checkTrue(cleanr::check_file(file))
}



test_s4_dircetory <- function() {
    directory <- system.file("runit_tests", "source", "R_s4",
                             package = "cleanr")
    RUnit::checkException(cleanr::check_directory(directory,
                                                  check_return = FALSE))
}

test_s4_package <- function() {
    package_path <- file.path(tempdir(), "fake")
    usethis::create_package(package_path, fields = NULL,
                            rstudio = FALSE, open = FALSE)
    directory <- system.file("runit_tests", "source", "R_s4",
                             package = "cleanr")
    file.copy(list.files(directory, full.names = TRUE), file.path(package_path,
                                                                  "R"))
    RUnit::checkTrue(cleanr::check_package(package_path, check_return = FALSE,
                                           max_line_width = 81))
}

.onLoad <- function(libname, pkgname) {
    if (is.character(libname) && is.character(pkgname)) {
       # soothe codetools::checkUsagePackage
    }
    set_cleanr_options(overwrite = FALSE)
    return(invisible(NULL))
}

#' get the body of a function
#'
#' \code{\link{body}} reformats the code (see \emph{Examples} and
#' \emph{Details}).
#'
#' If we want to check a function as it is coded in a file, we need to get the
#' original thing as coded in the file, no optimized version.
#' In \code{\link{sys.source}} and \code{\link{source}}, keep.source = FALSE
#' seems to use \code{\link{body}}.
#' So we have to do it all over after sourcing with keep.source = TRUE (in
#' \code{\link{check_functions_in_file}}).
#'
#' @param object The function from which to extract the body. \cr Should have
#' been sourced with keep.source = TRUE.
#' @return [character(n)]\cr the function body's lines.
#' @keywords internal
#' @examples
#' source(system.file("source", "R", "utils.R", package = "cleanr"))
#' require(checkmate)
#' cleanr:::get_function_body(set_cleanr_options)[3:6]
#' utils::capture.output(body(set_cleanr_options))[4:6]
get_function_body <- function(object) {
    checkmate::checkFunction(object)
    checkmate::checkFunction(object)
    captured_function <- utils::capture.output(object)
    # if the function is not defined in the global environment, the environment
    # will be added to capture.output()
    lines_in_function <- captured_function[! grepl("<environment:\\.*",
                                                   captured_function)]
    # remove the byte code
    lines_in_function <- lines_in_function[! grepl("<bytecode:\\.*",
                                                   lines_in_function)]
    if (!any(grepl("{", lines_in_function, fixed = TRUE))) {
        # treat oneliners
        is_split_onliner <- length(lines_in_function) > 1
        opening_line_num <- 1  + as.numeric(is_split_onliner)
        closing_line_num  <- length(lines_in_function)
    } else {
        opening_line_num <- min(grep("{", lines_in_function, fixed = TRUE))
        closing_line_num <- max(grep("}", lines_in_function, fixed = TRUE))
    }
    opening_brace_ends_line <-
        grepl("\\{\\s*$", lines_in_function[opening_line_num])
    if (opening_brace_ends_line)
        opening_line_num <- opening_line_num + 1
    closing_brace_starts_line <-
        grepl("^\\s*\\}", lines_in_function[closing_line_num])
    if (closing_brace_starts_line)
        closing_line_num <- closing_line_num - 1
    return(lines_in_function[opening_line_num:closing_line_num])
}

#' throw a condition
#'
#' throws a condition of class c("cleanr", "error", "condition").
#'
#' We use this condition as an error dedicated to \pkg{cleanr}.
#'
#' @keywords internal
#' @param message_string The message to be thrown.
#' @param system_call The call to be thrown.
#' @param ... Arguments to be passed to \code{\link{structure}}.
#' @return FALSE. But it does not return anything, it stops with a
#' condition of class c("cleanr", "error", "condition").
#' @keywords internal
#' @examples
#' tryCatch(cleanr:::throw("Hello error!"), cleanr = function(e) return(e))
throw <- function(message_string, system_call = sys.call(-1), ...) {
    checkmate::qassert(message_string, "s*")
    condition <- structure(
                           class = c("cleanr", "error",  "condition"),
                           list(message = message_string, call = system_call),
                           ...
                           )
    stop(condition)
}

#' tidy findings
#'
#' remove TRUE converted to class character from findings.
#'
#' \code{\link{check_directory}}, \code{\link{check_file}},
#' \code{\link{check_functions_in_file}} and
#' \code{\link{check_function_layout}} all collect tryCatch to collect either
#' TRUE for a check passed or a character holding a conditions message. This
#' function deletes the TRUES.
#'
#' @param findings a character vector with possibly some elements reading "TRUE"
#' or a vector of TRUES.
#' @return a character vector without any element reading "TRUE" or NULL.
#' @keywords internal
#' @examples
#' findings <- c("some signal caught", rep("TRUE", 3))
#' cleanr:::tidy_findings(findings)
tidy_findings <- function(findings) {
    if (is.logical(findings)) {
        ## findings may be all TRUE, so we set them NULL
        conditions <- NULL
    } else {
        ## findings are of class character with TRUE converted to "TRUE", so we
        ## remove these
        conditions <- findings[! findings == "TRUE"]
    }
    return(conditions)
}

if (interactive()) pkgload::load_all()
function_file <- system.file("runit_tests", "source", "R", "functions.R",
                             package = "cleanr")
source(function_file)
test_num_arguments <- function() {
    RUnit::checkTrue(cleanr::check_num_arguments(failing_function))
    RUnit::checkException(cleanr::check_num_arguments(failing_function,
                                                      max_num_arguments = 1))
}

test_check_return <- function() {
    RUnit::checkException(cleanr::check_return(failing_function))
    RUnit::checkTrue(cleanr::check_return(failing_function,
                                          check_return = FALSE))
}

test_check_nesting_depth <- function() {
    RUnit::checkException(cleanr::check_nesting_depth(failing_function))
    RUnit::checkTrue(cleanr::check_return(failing_function,
                                          check_return = FALSE))
}

test_num_lines <- function() {
    RUnit::checkException(cleanr::check_num_lines(failing_function,
                                                  max_lines = 9))

    RUnit::checkTrue(cleanr::check_num_lines(failing_function,
                                             max_lines = 10))
}

test_num_lines_of_code <- function() {
    RUnit::checkException(cleanr::check_num_lines_of_code(failing_function,
                                                          max_lines = 9))
    RUnit::checkTrue(cleanr::check_num_lines_of_code(failing_function,
                                                  max_lines = 10))
}

test_line_width <- function() {
    RUnit::checkException(cleanr::check_line_width(failing_function,
                                                          max_line_width = 9))
    RUnit::checkTrue(cleanr::check_line_width(failing_function,
                                                  max_line_width = 100))
}

test_file_width <- function() {
    RUnit::checkException(cleanr::check_file_width(function_file,
                                                          max_file_width = 9))
    RUnit::checkTrue(cleanr::check_file_width(function_file,
                                                  max_file_width = 100))
}

test_file_length <- function() {
    RUnit::checkException(cleanr::check_file_length(function_file,
                                                          max_file_length = 9))
    RUnit::checkTrue(cleanr::check_file_length(function_file,
                                                  max_file_length = 100))
}

#' @include internals.R
NULL

#' @include utils.R
NULL

#' Function Checks
#'
#' A set of tiny functions to check that functions adhere to a layout style.
#' A function should have a clear layout, it should
#' \itemize{
#'   \item not have too many arguments,
#'   \item not have too many levels of nesting,
#'   \item neither have too many lines nor
#'   \item have too many lines of code,
#'   \item not have lines too wide and
#'   \item explicitly \code{\link{return}} an object.
#' }
#'
#' In case of a fail all \code{\link{function_checks}} \code{\link{throw}} a
#' condition of class c("cleanr", "error", "condition").
#'
#' @section Warning: \code{\link{check_return}} just \code{\link{grep}}s for a
#' for a line starting with a \code{\link{return}} statement (ah, see the code
#' for the real thing).
#' This does not ensure that \emph{all} \code{\link{return}} paths from the
#' function are explicit and it may miss a \code{\link{return}} path after a
#' semicolon.
#' It just checks if you use \code{\link{return}} at all.
#'
#' @param object The function to be checked.
#' Should have been sourced with keep.source = TRUE (see
#' \code{\link{get_function_body}}).
#' @template return_invisibly_true_see_details
#' @name function_checks
#' @examples
#' print(cleanr::check_num_arguments(cleanr::check_num_arguments))
#' print(cleanr::check_nesting_depth(cleanr::check_nesting_depth))
#' print(cleanr::check_num_lines(cleanr::check_num_lines))
#' print(cleanr::check_num_lines_of_code(cleanr::check_num_lines_of_code))
#' print(cleanr::check_return(cleanr::check_return))
#' # R reformats functions on import (see
#' # help(get_function_body, package = "cleanr")), so we need 90 characters:
#' print(cleanr::check_line_width(cleanr::check_line_width,
#'                                max_line_width = 90))
NULL


#' @rdname function_checks
#' @param max_num_arguments The maximum number of arguments accepted.
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.
#' @export
#' @family check functions
check_num_arguments <- function(object,
                                max_num_arguments = gco("max_num_arguments")) {
    if (is_not_false(max_num_arguments, where = environment())) {
        checkmate::checkFunction(object)
        checkmate::qassert(max_num_arguments, "N1")
        num_arguments <- length(formals(object))
        if (num_arguments > max_num_arguments)
            throw(paste("found", num_arguments,
                        "arguments, max_num_arguments was", max_num_arguments))
    }
    return(invisible(TRUE))
}

#' @rdname function_checks
#' @param max_nesting_depth The maximum nesting depth accepted.
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.
#' @export
check_nesting_depth <- function(object,
                                max_nesting_depth = gco("max_nesting_depth")) {
    if (is_not_false(max_nesting_depth, where = environment())) {
        checkmate::checkFunction(object)
        checkmate::qassert(max_nesting_depth, "N1")
        function_body <- get_function_body(object)
        # break if no braces in function
        if (!any(grepl("}", function_body, fixed = TRUE)))
            return(invisible(TRUE))
        braces <- paste(gsub("\\", "",
                             gsub("[^\\{\\}]", "", function_body),
                             fixed = TRUE),
                        collapse = "")
        consectutive_openings <- strsplit(braces, "}",
                                          fixed = TRUE)[[1]]
        nesting_depths <- nchar(consectutive_openings)
        nesting_depth <- max(nesting_depths)
        if (nesting_depth > max_nesting_depth)
            throw(paste0("found nesting depth ", nesting_depth,
                         ", max_nesting_depth was ", max_nesting_depth))
    }
    return(invisible(TRUE))
}

#' @rdname function_checks
#' @param max_lines The maximum number of lines accepted.
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.

#' @export
check_num_lines <- function(object, max_lines = gco("max_lines")) {
    if (is_not_false(max_lines, where = environment())) {
        checkmate::checkFunction(object)
        checkmate::qassert(max_lines, "N1")
        function_body <- get_function_body(object)
        num_lines  <- length(function_body)
        if (num_lines > max_lines)
            throw(paste("found", num_lines, "lines, max_lines was", max_lines))
    }
    return(invisible(TRUE))
}

#' @rdname function_checks
#' @param max_lines_of_code The maximum number of lines of code accepted.
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.
#' @export
check_num_lines_of_code <- function(object,
                                    max_lines_of_code =
                                        gco("max_lines_of_code")) {
    if (is_not_false(max_lines_of_code, where = environment())) {
        checkmate::checkFunction(object)
        checkmate::qassert(max_lines_of_code, "N1")
        function_body <- get_function_body(object)
        line_is_comment_pattern <- "^\\s*#"
        lines_of_code <- grep(line_is_comment_pattern, function_body,
                              value = TRUE, invert = TRUE)
        num_lines_of_code <-  length(lines_of_code)
        if (num_lines_of_code > max_lines_of_code)
            throw(paste("found", num_lines_of_code,
                        "lines of code, max_lines_of_code was",
                        max_lines_of_code))
    }
    return(invisible(TRUE))
}

#' @rdname function_checks
#' @param max_line_width The maximum line width accepted.
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.
#' @export
check_line_width <- function(object,
                             max_line_width = gco("max_line_width")) {
    if (is_not_false(max_line_width, where = environment())) {
        checkmate::checkFunction(object)
        checkmate::qassert(max_line_width, "N1")
        function_body <- get_function_body(object)
        line_widths <-  nchar(function_body)
        if (any(line_widths > max_line_width)) {
            long_lines_index <- line_widths > max_line_width
            long_lines <- seq(along = function_body)[long_lines_index]
            throw(paste("line ", long_lines, ": found width ",
                        line_widths[long_lines_index], " max_line_width was ",
                        max_line_width,
                        sep = "", collapse = "\n")
            )
        }
    }
    return(invisible(TRUE))
}

#' @rdname function_checks
#' @param check_return
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.
#' @export
check_return <- function(object,
                         check_return = gco("check_return")) {
    if (is_not_false(check_return, where = environment())) {
        checkmate::checkFunction(object)
        message_string <- paste("Just checking for a line starting with a ",
                                "return statement.\n",
                                "This is no check for all return paths ",
                                "being explicit.")
        warning(message_string)
        checkmate::checkFunction(object)
        function_body <- body(object)  # body gives us the statements line
        # by line, some_command;   return(somewhat) will be matched
        # by: ^\\s*return\\(
        if (! any(grepl("^\\s*return\\(", function_body)))
            throw("found no return() statement at all.")
    }
    return(invisible(TRUE))
}

#' File Checks
#'
#' A set of tiny functions to check that files adhere to a layout style.
#' A file should have a clear layout, it should
#' \itemize{
#'   \item not have too many lines and
#'   \item not have lines too wide.
#' }
#'
#' In case of a fail all \code{\link{file_checks}} \code{\link{throw}} a
#' condition of class c("cleanr", "error", "condition").
#'
#'
#' @param path Path to the file to be checked.
#' @return \code{\link[base:invisible]{Invisibly}} \code{\link{TRUE}},
#' but see \emph{Details}.
#' @name file_checks
#' @family check functions
#' @examples
#' print(cleanr::check_file_width(system.file("source", "R", "checks.R",
#'                                            package = "cleanr")))
#' print(cleanr::check_file_length(system.file("source", "R", "checks.R",
#'                                             package = "cleanr"),
#'                                             max_file_length = 300))
NULL

#' @rdname file_checks
#' @param max_file_width The maximum line width accepted.
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.
#' @export
check_file_width <- function(path,
                             max_file_width = gco("max_file_width")) {
    if (is_not_false(max_file_width, where = environment())) {
        checkmate::qassert(path, "S1")
        checkmate::qassert(max_file_width, "N1")
        file_content <- readLines(path)
        line_widths <-  nchar(file_content)
        if (any(line_widths > max_file_width)) {
            long_lines_index <- line_widths > max_file_width
            throw(paste0(path, ": line ",
                         seq(along = file_content)[long_lines_index],
                         " counts ", line_widths[long_lines_index],
                         " characters.",
                         collapse = "\n")
            )
        }
    }
    return(invisible(TRUE))
}

#' @rdname file_checks
#' @param max_file_length The maximum number of lines accepted.
#' Set (preferably via \code{\link{set_cleanr_options}}) to \code{NULL} or
#' \code{FALSE} to disable the check.
#' @export
check_file_length <- function(path,
                              max_file_length = gco("max_file_length")) {
    if (is_not_false(max_file_length, where = environment())) {
        checkmate::qassert(path, "S1")
        checkmate::qassert(max_file_length, "N1")
        file_content <- readLines(path)
        num_lines <- length(file_content)
        if (num_lines > max_file_length) {
            throw(paste0(path, ": ", num_lines, " lines in file.",
                         collapse = "\n"))
        }
    }
    return(invisible(TRUE))
}

failing_function <- function(x, clean = FALSE) {
    y <- c(x, x)  # add some comment that makes the line toooooooooooooooooooooooo wide
    # this is a comment only
    if (istrue(clean)) {
        if (istrue(clean)) {
            if (istrue(clean)) {
                if (istrue(clean)) {
                    message("This is nested")
                }
            }
        }
    }
}

passing_function <- function(x, clean = FALSE) {
    y <- c(x, x)
    # this is a comment only
    if (istrue(clean)) {
        if (istrue(clean)) {
            message("This is nested")
        }
    }
    return(TRUE)
}

#' @include checks.R
NULL

#' Check a Function's Layout
#'
#' Run all \code{\link{function_checks}} on a function.
#'
#' The functions catches the messages of "cleanr"-conditions
#' \code{\link{throw}}n by \code{\link{function_checks}} and, if it caught any,
#' \code{\link{throw}}s them.
#'
#' @param object The function to be checked.
#' @param function_name The name to be used for reporting. Stick with the
#' default: If NULL, it is taken from the \code{object} given.
#' Argument is used internally to pass function names
#' retrieved via \code{\link{get}} in the wrapper function
#' \code{\link{check_functions_in_file}}.
#' @param max_lines_of_code See \code{\link{check_num_lines_of_code}}.
#' @param max_lines See \code{\link{check_num_lines}}.
#' @param max_num_arguments See \code{\link{check_num_arguments}}.
#' @param max_nesting_depth See \code{\link{check_nesting_depth}}.
#' @param max_line_width See \code{\link{check_line_width}}.
#' @param check_return See \code{\link{check_return}}.
#' @template return_invisibly_true_see_details
#' @export
#' @family wrappers
#' @examples
#' print(cleanr::check_function_layout(cleanr::check_num_lines))
check_function_layout <- function(object, function_name = NULL,
                                  max_lines_of_code =
                                  get_cleanr_options("max_lines_of_code"),
                                  max_lines = get_cleanr_options("max_lines"),
                                  max_num_arguments =
                                  get_cleanr_options("max_num_arguments"),
                                  max_nesting_depth =
                                  get_cleanr_options("max_nesting_depth"),
                                  max_line_width =
                                  get_cleanr_options("max_line_width"),
                                  check_return =
                                  get_cleanr_options("check_return")
                                  ) {
    findings <- NULL
    finding <- tryCatch(check_num_arguments(object,
                                   max_num_arguments = max_num_arguments),
                          cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    finding <- tryCatch(check_nesting_depth(object,
                                   max_nesting_depth = max_nesting_depth),
                          cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    finding <- tryCatch(check_line_width(object,
                                   max_line_width = max_line_width),
                          cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    finding <- tryCatch(check_num_lines(object,
                                   max_lines = max_lines),
                          cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    finding <- tryCatch(check_num_lines_of_code(object,
                                       max_lines_of_code = max_lines_of_code),
                          cleanr = function(e) return(e[["message"]]))

    findings <- c(findings, finding)
    finding <- tryCatch(check_return(object, check_return = check_return),
                          cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    findings <- tidy_findings(findings)
    if (! is.null(findings)) {
        if (is.null(function_name)) {
            function_name <- deparse(substitute(object))
        }
        throw(paste(function_name, names(findings),
                    findings, sep = " ", collapse = "\n"))
    }
    return(invisible(TRUE))
}

#' Check a File's Layout
#'
#' Run all \code{\link{file_checks}} on a file.
#'
#' The function catches the messages of "cleanr"-conditions \code{\link{throw}}n
#' by \code{\link{file_checks}} and, if it caught any, \code{\link{throw}}s
#' them.
#'
#' @param path Path to the file to be checked.
#' @param max_file_length See \code{\link{check_file_length}}.
#' @param max_file_width See \code{\link{check_file_width}}.
#' @template return_invisibly_true_see_details
#' @family wrappers
#' @export
#' @examples
#' print(cleanr::check_file_layout(system.file("source", "R", "checks.R",
#'                                             package = "cleanr")))
check_file_layout <- function(path,
                              max_file_length =
                                  get_cleanr_options("max_file_length"),
                              max_file_width =
                                  get_cleanr_options("max_file_width")) {
    findings <- NULL
    finding <- tryCatch(check_file_width(path,
                                   max_file_width = max_file_width),
                          cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    finding <- tryCatch(check_file_length(path,
                                   max_file_length = max_file_length),
                          cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    findings <- tidy_findings(findings)
    if (! is.null(findings)) {
        throw(paste0(findings, collapse = "\n"))
    }
    return(invisible(TRUE))
}

#' Check All Functions Defined in a File
#'
#' Run \code{\link{check_function_layout}} on all functions defined in a file.
#'
#' The functions catches the messages of "cleanr"-conditions
#' \code{\link{throw}}n by \code{\link{check_function_layout}} and,
#' if it caught any, \code{\link{throw}}s them.
#'
#' @param path Path to the file to be checked.
#' @param ... Arguments to be passed to \code{\link{check_function_layout}}.
#' @template return_invisibly_true_see_details
#' @family wrappers
#' @export
#' @examples
#' print(cleanr:::check_functions_in_file(system.file("source", "R", "utils.R",
#'                                                    package = "cleanr")))
check_functions_in_file <- function(path, ...) {
    checkmate::assertFile(path, access = "r")
    findings <- NULL
    source_kept <- new.env(parent = globalenv())
    sys.source(path, envir = source_kept, keep.source = TRUE)
    for (name in ls(envir = source_kept, all.names = TRUE)) {
        assign(name, get(name, envir = source_kept))
        if (is.function(get(name))) {
            finding <-
                tryCatch(check_function_layout(get(name,
                                                   envir = source_kept),
                                               function_name = name, ...),
                         cleanr = function(e) return(e[["message"]]))
            findings <- c(findings, finding)
        }
    }
    findings <- tidy_findings(findings)
    if (! is.null(findings)) {
        throw(paste(path, names(findings),
                    findings, sep = " ", collapse = "\n"))
    }
    return(invisible(TRUE))
}

#' Check a File
#'
#' Run \code{\link{check_functions_in_file}} and
#' \code{\link{check_file_layout}} on a file.
#'
#' The function catches the messages of "cleanr"-conditions \code{\link{throw}}n
#' by \code{\link{check_functions_in_file}} and \code{\link{check_file_layout}}
#' and, if it
#' caught any, \code{\link{throw}}s them.
#'
#' @param path Path to the file to be checked.
#' @param ... Arguments to be passed to \code{\link{check_functions_in_file}} or
#' \code{\link{check_file_layout}}.
#' @template return_invisibly_true_see_details
#' @family wrappers
#' @export
#' @examples
#' print(cleanr::check_file(system.file("source", "R", "utils.R",
#'                                      package = "cleanr")))
check_file <- function(path, ...) {
    checkmate::assertFile(path, access = "r")
    findings <- NULL
    # I know of two ways to pass arguments through a wrapper to different
    # functions: ellipsis and explicit arguments. I've used ellipsis here, to
    # avoid using ellipsis eating unused arguments down the line, I filter the
    # ellpsis. This is quite a massacre.
    # TODO: refactor with named list as argument containers for functions, i.e.
    # arguments (path, check_file_layout_args = list(...), ...).
    dots <- list(...)
    check_file_layout_defaults <- formals(check_file_layout)
    check_functions_defaults <- append(formals(check_functions_in_file),
                                            formals(check_function_layout))
    known_defaults <- append(check_file_layout_defaults,
                             check_functions_defaults)
    if (! all(names(dots) %in% names(known_defaults))) {
        stop(paste("got unkown argument(s): ",
                   paste(names(dots)[! names(dots) %in% names(known_defaults)],
                         collapse = ", ")))
    }
    arguments <- append(list(path = path), dots)

    use <- utils::modifyList(check_file_layout_defaults, arguments,
                             keep.null = TRUE)
    arguments_to_use <- use[names(use) %in% names(check_file_layout_defaults)]
    # use only non-empty arguments
    arguments_to_use <- arguments_to_use[arguments_to_use != ""]
    finding <- tryCatch(do.call("check_file_layout", arguments_to_use),
                        cleanr = function(e) return(e[["message"]]))
    findings <- c(findings, finding)
    use <- utils::modifyList(check_functions_defaults, arguments,
                             keep.null = TRUE)
    arguments_to_use <- use[names(use) %in%
                            names(check_functions_defaults)]
    # use only non-empty arguments
    arguments_to_use <- arguments_to_use[arguments_to_use != ""]
    # TODO: I remove function_name to keep it from being passed via the ellipsis
    arguments_to_use <- arguments_to_use[names(arguments_to_use) !=
                                         "function_name"]
    finding <- tryCatch(do.call("check_functions_in_file", arguments_to_use),
                        cleanr = function(e) return(e[["message"]]),
                        error = function(e) return(e))
    if (inherits(finding, "error") && !inherits(finding, "cleanr")) {
        finding[["message"]] <- paste0(finding[["message"]],
                                       ". Probably due to failed ",
                                       "S4 method loading.")
        stop(finding)
    }
    findings <- c(findings, finding)
    findings <- tidy_findings(findings)
    if (! is.null(findings)) {
        throw(paste(names(findings),
                    findings, sep = " ", collapse = "\n"))
    }
    return(invisible(TRUE))
}

#' Check a Directory
#'
#' Run \code{\link{check_file}} on files in a directory.
#'
#' The function catches the messages of "cleanr"-conditions
#' \code{\link{throw}}n by \code{\link{check_file}} and, if it caught any,
#' \code{\link{throw}}s them.
#'
#' @param path Path to the directory to be checked.
#' @param pattern A pattern to search files with, see \code{\link{list.files}}.
#' @param recursive Search the directory recursively?
#' See \code{\link{list.files}}.
#' @param ... Arguments to be passed to \code{\link{check_file}}.
#' @template return_invisibly_true_see_details
#' @seealso \code{\link{check_package}}.
#' @export
#' @family wrappers
#' @examples
#' # load internal functions first.
#' load_internal_functions("cleanr")
#' print(cleanr::check_directory(system.file("source", "R", package = "cleanr"),
#'                               max_num_arguments = 8, max_file_width = 90,
#'                               max_file_length = 350,
#'                               check_return = FALSE))
check_directory <- function(path, pattern = "\\.[rR]$", recursive = FALSE,
                            ...) {
    checkmate::assertDirectory(path, access = "r")
    paths <- normalizePath(sort(list.files(path, pattern, recursive = recursive,
                                           full.names = TRUE)))
    findings <- NULL
    for (source_file in paths) {
        finding <- tryCatch(check_file(source_file, ...),
                            cleanr = function(e) return(e[["message"]]))
        findings <- c(findings, finding)
    }
    findings <- tidy_findings(findings)
    if (! is.null(findings)) {
        throw(paste(path, names(findings),
                    findings, sep = " ", collapse = "\n"))
    }
    return(invisible(TRUE))
}

#' Check a Package
#'
#' Run \code{\link{check_file}} on a package's source.
#'
#' The function catches the messages of "cleanr"-conditions
#' \code{\link{throw}}n by \code{\link{check_file}} and, if it caught any,
#' \code{\link{throw}}s them.
#'
#' @param path Path to the package to be checked.
#' @param pattern A pattern to search files with, see \code{\link{list.files}}.
#' @param ... Arguments to be passed to \code{\link{check_file}}.
#' @template return_invisibly_true_see_details
#' @export
#' @family wrappers
#' @examples
#' # create a fake package first:
#' package_path <- file.path(tempdir(), "fake")
#' usethis::create_package(package_path, fields = NULL,
#'                         rstudio = FALSE, open = FALSE)
#' directory <- system.file("runit_tests", "source", "R_s4",
#'                          package = "cleanr")
#' file.copy(list.files(directory, full.names = TRUE), file.path(package_path,
#'                                                               "R"))
#' RUnit::checkTrue(cleanr::check_package(package_path, check_return = FALSE))
check_package <- function(path, pattern = "\\.[rR]$", ...) {
    checkmate::assertDirectory(path, access = "r")
    package_root <- rprojroot::find_root(rprojroot::is_r_package, path)
    pkgload::load_all(package_root)
    paths <- normalizePath(sort(list.files(file.path(package_root, "R"),
                                           pattern = pattern,
                                           full.names = TRUE)))
    findings <- NULL
    for (source_file in paths) {
        finding <- tryCatch(check_file(source_file, ...),
                            cleanr = function(e) return(e[["message"]]))
        findings <- c(findings, finding)
    }
    findings <- tidy_findings(findings)
    if (! is.null(findings)) {
        throw(paste(path, names(findings),
                    findings, sep = " ", collapse = "\n"))
    }
    return(invisible(TRUE))
}
